div#editor {
    position: absolute;
    display: none;
    width: 540px;
    padding: 10px;
    background: #cdcdcd; 
    color: #333;
    border: #30677c 1px solid;
    border-radius: 15px;
    min-height: 100px;
    text-align: left;
}
div#editorheader {
    width:100%;
    background: #30677c;
    text-align: center;
    font-weight: bold;
    color: white;
    border-radius: 5px;
    margin-top: -3px;
    cursor: move;
}
div#editorheader button {
    float: right;
    margin-top: -5px;
    background: #30677c;
    cursor: pointer;
}
div#editor table {
    width: 100%;
}
div#editor th {
    font-weight: bold;
    overflow: hidden;
    max-width: 150px;
    vertical-align: top;
    border-bottom: 1px #eee solid;
}
div#editor tr:hover th {
    color: #30677c;
}
div#editor th span.setname {
    font-weight: normal;
    font-size: 8pt;
    padding-left: 10px;
}
div#editor td {
    width: 70%;
    font-style: italic;
    border-bottom: 1px #eee solid;
}
div#editor tr.focus {
    background: #aaa;
}
div#editor tr.focus span.editforms button {
    background: #777;
}
div#editor tr.focus span.editforms button.selected {
    background: #30677c;
}
div#editor div#id {
    width: 100%;
    text-align: center;
    font-size: 8px;
    font-style: italic;
    color: #666;
}
div#editor .setname {
    font-size: 8px;
    font-style: italic;
}
div#editor .buttoncontainer span {
    padding-left: 10px;
    padding-right: 10px;
}
div#editor input.textedit, div#editor select.textedit, div#editor .text {
    color: #333;
    font-style: italic;
    max-width: 280px;
}
div#editor input.classedit, div#editor select.classedit, div#editor .class {
    color: #3323cd;
    max-width: 280px;
}
div#editor input.subsetedit, div#editor select.subsetedit, div#editor .subset {
    color: #6d0000;
    max-width: 280px;
}
input.changed {
    color: #660000;
}
div#editor .addhigherordermenu {
    float: right;
    border-radius: 0;
    background: transparent;
    border: 1px solid #30677c;
    color: #30677c;
    padding: 1px;
    clear: both;
    cursor: pointer;
}
.addhigherordermenu ul {
    display: none;
}
.addhigherordermenu:hover ul {
    display: block;
    margin-top: -10px;
    position: absolute;
    min-width: 160px;
    display: block;
    background: black;
    padding: 5px;
    padding-left: 10px;
    opacity: 0.9;
}
.addhigherordermenu li {
    display: block;
    list-style-type: none;
    padding-left: 5px;
    padding-right: 5px;
    color: white;
    font-weight: bold;
    cursor: pointer;
}
.addhigherordermenu li:hover {
    text-decoration: underline;
    cursor: pointer;
}
.higherorderrow {
    vertical-align: top;
}
.higherorderrow textarea {
    width: 100%;
}

div.selected span.lbl {
    background: yellow;
    color: black;
}
#editor button.spanselector {
    background: #444;
    font-size: 80%;
    float: right;
}
#editor button.selectoron {
    background: yellow;
    color: black;
}
div.queued span.lbl {
    background: red;
    color: yellow;
}
#editoraddfields, #editortargets {
    display: block;
}
div#editformcorrectionsetselector {
    display: none;
    width: 160px;
    background: transparent;
    font-size: 9px;
}
#editortargets label, #editortargetlist {
    display: none;
}
div#newdeclaration, div#history, div#console {
    position: absolute;
    display: none;
    width: 520px;
    padding: 10px;
    background: #cdcdcd; 
    color: #333;
    border: #30677c 1px solid;
    border-radius: 15px;
    min-height: 100px;
    text-align: left;
    left: 30%;
}
div#newdeclarationheader, div#historyheader, div#consoleheader {
    width:100%;
    background: #30677c;
    text-align: center;
    font-weight: bold;
    color: white;
    border-radius: 5px;
    margin-top: -3px;
}
div#newdeclaration button, div#history button, div#consoleheader button {
    float: right;
    margin-top: -5px;
    background: #30677c;
}
#newdeclarationset {
    width: 75%;
}
span.editforms {
    display: inline-block;
    white-space: nowrap;
}
span.editforms form {
    display: inline;
}
span.editforms button {
    min-width: 12px;
    min-height: 12px;
    background: #aaa;
}
span.editforms button.selected {
    background: #30677c;
}
#submitoptionscontainer {
    font-size: 60%;
    color: #666;
    text-align: left;
}
#console textarea {
    width: 90%;
    min-height: 320px;
}
div#qunit {
    width: 60%;
    margin-left: auto;
    margin-right: auto;
}

#editor div.suggestions label {
    font-size: 80%;
}
#editsuggestions {
    color: #0f5722;
    background: #aaa;
}
#saveversion {
    width: 100px;
    font-size: 80%;
    position: fixed;
    right: 10px;
    top: 150px;
}
#versionlabel {
    width: 95%;
}
#saveversion label {
    display: block;
}
#saveversion input {
    display: block;
}
#saveversion button {
    float: right;
}
div#queueinfo {
    width: 100px;
    position: fixed;
    right: 10px;
    top: 75px;
    background: transparent;
}
span.repeatnotice {
    color: yellow;
    font-size: 80%;
}
div.confidenceeditor {
    width: 99%;
}
div.confidenceeditor div {
    display: inline-block;
    width: 60%;
    font-size: 80%;
    font-style: italic;
}
#notice {
    display: none;
    width: 150px;
    position: fixed;
    right: 20px;
    top: 250px;
    background: #d3d498;
    color: black;
    padding: 10px;
    font-weight: bold;
    border: 2px dashed black;
    text-align: center;
}

.parentspaneditor select {
    max-width: 320px;
    overflow: hidden;
}

