#metadata h2 {
    text-align: center;
    margin-left: auto;
    margin-right: auto;
}
#metadata table {
    width: 60%;
    margin-left: auto;
    margin-right: auto;
    border: 1px solid #416887;
    background: #b3d0db;
}
#metadata .key {
    width: 29%;
}
#metadata .key  input {
    color: #416887;
    font-style: italic;
    font-weight: bold;
}
#metadata .value {
    width: 69%;
}
#metadata .value  input {
    font-style: italic;
}
#metadata table input {
    width: 95%;
}
#metadata .buttons {
    margin-top: 20px;
    margin-bottom: 20px;
    text-align: center;
    margin-left: auto;
    margin-right: auto;
}
