{% extends "base.html" %}

{% block head %}
<!-- dependencies -->
{% endblock %}

{% block menu %}
{% endblock %}

{% block dialogs %}
{% endblock %}

{% block main %}
<div id="metadata">
</div>
{% endblock %}

