<div id="treeview">
    <div id="treeviewheader">
        <span>Tree Viewer</span>
        <button id="treeviewdiscard">x</button>
    </div>
    <div class="body">
        <svg id="tree"></svg>
    </div>
</div>
