body {
    font-family: Ubuntu, Verdana, sans-serif;
    font-size: 10pt;
    background: #cbd9de;
    margin: 0px;
    padding: 0px;
}

#topbar {
    width: 100%;
    margin: 0;
    padding: 5px;
    background: black;
    color: white;
    min-height: 12pt;
    text-align: center;
    font-size: 90%;
    cursor: pointer;
}
#topbar ul {
    margin: 0px;
    padding: 0px;
}
#topbar li {
    display: inline;
    list-style-type: none;
    padding-left: 20px;
    padding-right: 20px;
}
#topbar li:hover {
    background: #444;
}
#topbar .left {
    float: left;
    margin-left: 20px;
}
#topbar .right {
    float: right;
    margin-right: 20px;
}

#topbar a {
    color: white;
    font-weight: bold;
    text-decoration: none;
}
#topbar nav a {
    color: white;
    font-weight: normal;
}
#topbar a:hover {
    color: #cbd9de;
    text-decoration: underline;
}

div.dialog {
    margin-top: 50px;
    margin-left: auto;
    margin-right: auto;
    width: 50%;
    padding: 10px;
    background: #ededed;
    color: #333;
    border: #30677c 1px solid;
    border-radius: 15px;
    min-height: 100px;
}
div.dialog label {
    display: inline-block;
    font-weight: bold;
    width: 20%;
    margin: 5px;
}
div.dialog input[type="input"], div.dialog select {
    display: inline-block;
    width: 70%;
    background: white;
    border: #30677c 1px solid;
    border-radius: 5px;
    padding: 2px;
    margin: 5px;
}
button, .button, div.dialog input.button {
    background: #2a2a2a;
    color: white;
    font-weight: bold;
    text-align: center;
    border: none;
    border-radius: 10px;
    text-align: center;
    padding: 5px;
    cursor: pointer;
}
div.error {
    color: red;
    font-weight: bold;
    border: #220000 dashed 1px;
    padding: 5px;
    text-align: center;
}
div.buttoncontainer {
    padding: 5px;
    text-align: center;
}
#topbar span.title {
    font-weight: bold;
    text-align: center;
    color: #cbd9de;
}
#topbar span.title img {
    height: 12pt;
}
h3 {
    border-bottom: #30677c 1px solid;
    color: #30677c;
    margin-top: -5px;
}
ul#doclist,ul#dirlist {
    border: #30677c 1px solid;
    border-radius: 5px;
    padding: 2px;
    margin: 5px;
    background: white;
    color: black;
    min-height: 50px;
    list-style: none;
    clear: both;
}
ul#dirlist>li {
    font-weight: bold;
}
ul#doclist>li, ul#dirlist>li {
    background: #eee;
    border-top: 1px #fff solid;
    border-bottom: 1px #ccc solid;
    font-weight: normal;
    padding-left: 20px;
    padding-top: 3px;
    padding-bottom: 3px;
}
ul#doclist>li:hover, ul#dirlist>li:hover {
    background: #ddd;
}
#doclist span {
    float: right;
}
a {
    color: #30677c;
    text-decoration: none;
}

a:hover {
    text-decoration: underline;
}
#topbar nav ul ul {
    display: none;
    margin: 0px;
    padding: 0px;
    opacity: 0.9;
    z-index: 100;
}
#topbar nav ul li:hover > ul {
    position: absolute;
    min-width: 160px;
    display: block;
    background: black;
    padding: 5px;
    padding-left: 10px;
    opacity: 0.9;
}
#topbar nav ul li:hover > ul li {
    display: block;
    padding: 5px;
    padding-left: 10px;
    padding-right: 10px;
    overflow-wrap: break-word;
    word-wrap: break-word;
}
#topbar nav li li {
    display: inline-block;
    list-style-type: none;
    padding-left: 5px;
    padding-right: 5px;
}
nav>ul>li {
    font-weight: bold;
}


nav ul ul li.on {
    background: #0b470a;
}



#document {
    margin-left: auto;
    margin-right: auto;
    width: 60%;
    color: #222;
    background: #eee;
    border: #30677c 1px solid;
    border-radius: 25px;
    padding: 25px;
    margin-top: 20px;
    margin-bottom: 200px;
    min-width: 480px;
    min-height: 480px;
    max-width: 900px;
}
#wait {
    position: fixed;
    background: black;
    width: 100%;
    min-width: 100%;
    height: 100%;
    min-height: 100%;
    opacity: 0.8;
    text-align: center;
    padding-top: 200px;
    left: 0px;
    top: 0px;
}
#wait span.msg {
    display: block;
    text-align: center;
    margin-left: auto;
    margin-right: auto;
    margin-bottom: 20px;
    width: 80%;
    color: white;
    font-size: 20px;
}
#doclist span {
    font-size: 8px;
    font-family: monospace;
    margin-left: 20px;
}

.indexbuttoncontainer {
    float: right;
    margin-bottom: 4px;
}
#uploadform {
    display: none;
}
#adddirectoryform {
    display: none;
}
#footer {
    width: 80%;
    margin-left: auto;
    margin-right: auto;
    font-family: monospace;
    font-size: 80%;
    text-align: center;
    margin-top: 100px;
}
img#logo {
    display: block;
    width: 10%;
    margin-left: auto;
    margin-right: auto;
    margin-top: 25px;
}
span.helptext {
    display: block;
}
#aborted {
    display: none;
    margin-top: -100px;
}
#aborted, #fatalerror {
    width: 50%;
    color: #aa0000;
    border: 2px dashed #aa0000;
    padding: 30px;
    margin-left: auto;
    margin-right: auto;
    text-align: center;
    font-size: 110%;
}
#aborted strong, #fatalerror strong {
    font-size: 110%;
}
#aborted span.help, #fatalerror span.help {
    font-size: 70%;
    font-style: italic;
}
#connectioninfo {

}
div#leftpane {
    position: fixed;
    width: 160px;
    left:10px;
    top: 50px;
    font-size: 9px;
    overflow-x: hidden;
}
div#perspective, div#selector {
    width: 160px;
}
div#textclassselector {
    display: none;
}
div#leftpane .title {
    display: block;
    font-size: 10px;
    font-weight: bold;
    font-style: italic;
    color: black;
    text-align: left;
}
#perspectivemenu, #selectormenu {
    max-width: 160px;
}
#convparameterhelp, #convparameters {
   display: none;
}
#filemanagement {
    border: #30677c 1px solid;
    border-radius: 5px;
    padding: 2px;
    margin: 5px;
    color: black;
    clear: both;
    font-size: 80%;
    padding-left: 30px;
    padding-bottom: 30px;
}
